/* Embedded OS Loader
 *
 *   Copyright 2005 Vivien Chappelier <vivien.chappelier@free.fr>
 *
 * config file parsing
 *
 */

#include "stdlib.h"
#include "list.h"
#include "conf.h"
#include "icon.h"

static void conf_init(struct conf *conf)
{
	conf->timeout = 0;
	list_init(&conf->entries);
}

static void conf_add(struct conf *conf, int icon,
		     char *title, char *kernelname, char *initrdname)
{
	struct confentry *entry;

	entry = (struct confentry *) malloc(sizeof(struct confentry));
	entry->title = title;
	entry->icon = icon;
	entry->kernelname = kernelname;
	entry->initrdname = initrdname;

	list_add(&conf->entries, &entry->list);
}

#define MAX_LINE_SIZE 256

static int conf_getline(char *input, char buf[MAX_LINE_SIZE], int size)
{
	char c;
	char *line = buf;
	int len = 0;
	int comment = 0;
	int equals = 0;

	while (len < size) {
		c = *input++;
		switch (c) {
			/* end of line: return */
		case '\r':
		case '\n':
			len++;
		case '\0':
			*line = '\0';
			return len;

			/* space: ignore before = */
		case ' ':
		case '\t':
			len ++;
			if (equals)
				*line++ = c;
			break;

			/* equal sign: toggle space ignore */
		case '=':
			len ++;
			*line++ = c;
			equals = 1;
			break;

			/* comment: ignore end of line */
		case '#':
			len++;
			comment = 1;
			break;

			/* other characters */
		default:
			len++;
			if (!comment)
				*line++ = c;
			break;
		}

		if (line - buf >= MAX_LINE_SIZE - 1) {
			buf[MAX_LINE_SIZE - 1] = 0;
			return len;
		}
	}

	return len;
}

int str2icon(char *str)
{
	int icon = ICON_NONE;

	if (!__strncmp(str, "TUX", sizeof("TUX") - 1)) {
		icon = ICON_TUX;
	} else if (!__strncmp(str, "WXP", sizeof("WXP") - 1)) {
		icon = ICON_WINDOWS;
	}

	return icon;
}

int conf_parse(struct conf *conf, char *confdata, int size)
{
	int ret;
	char line[MAX_LINE_SIZE];
	char *image = NULL;
	char *label = NULL;
	char *initrd = NULL;
	int icon = ICON_NONE;

	conf_init(conf);

	while ((ret = conf_getline(confdata, line, size)) > 0) {
		confdata += ret;
		size -= ret;

		/* skip empty lines */
		if (line[0] == '\0')
			continue;

		printb("line: %d %s\n", ret, line);

		/* parse */
		if (!__strncmp(line, "image=", sizeof("image=") - 1)) {
			/* finish the previous image */
			if (image != NULL) {
				if (label == NULL)
					label = "No label";

				conf_add(conf, icon, label, image, initrd);
			}

			image = strdup(line + sizeof("image=") - 1);
			label = NULL;
			initrd = NULL;
			icon = ICON_NONE;

		} else if (!__strncmp(line, "label=", sizeof("label=") - 1)) {
			label = strdup(line + sizeof("label=") - 1);
		} else if (!__strncmp(line, "initrd=", sizeof("initrd=") - 1)) {
			initrd = strdup(line + sizeof("initrd=") - 1);
		} else if (!__strncmp(line, "icon=", sizeof("icon=") - 1)) {
			icon = str2icon(line + sizeof("icon=") - 1);
		}
	}

	/* finish the previous image */
	if (image != NULL) {
		if (label == NULL)
			label = "No label";
		
		conf_add(conf, icon, label, image, initrd);
	}

	return 0;
}
