/* Embedded OS Loader
 *
 *   Copyright 2005 Vivien Chappelier <vivien.chappelier@free.fr>
 *
 * keypad routines
 *
 */

#include "fb.h"
#include "keypad.h"
#include "stdlib.h"

#define OMAP_MPUIO_BASE      0xfffb5000
#define OMAP_MPUIO_KBC       0x0a
#define OMAP_MPUIO_KBR_LATCH 0x08

struct kp_key {
        unsigned int code_lo;
        unsigned int code_hi;
        unsigned int key;
};

#define MULTIKEY(hi, lo, val) { .code_hi = hi, .code_lo = lo, .key = val }

static struct kp_key keymap[] = {
        MULTIKEY(0x00000000, 0x00000003, KEY_LEFT),
        MULTIKEY(0x00000000, 0x0000000c, KEY_RIGHT),
        MULTIKEY(0x00000000, 0x00000005, KEY_UP),
        MULTIKEY(0x00000000, 0x0000000a, KEY_DOWN),
        MULTIKEY(0x00000000, 0x00000100, KEY_1),
        MULTIKEY(0x00000000, 0x00010000, KEY_2),
        MULTIKEY(0x00000000, 0x01000000, KEY_3),
        MULTIKEY(0x00000001, 0x00000000, KEY_4),
        MULTIKEY(0x00000000, 0x00000200, KEY_5),
        MULTIKEY(0x00000000, 0x00020000, KEY_6),
        MULTIKEY(0x00000000, 0x02000000, KEY_7),
        MULTIKEY(0x00000002, 0x00000000, KEY_8),
        MULTIKEY(0x00000000, 0x00000400, KEY_9),
        MULTIKEY(0x00000000, 0x00040000, KEY_0),
        MULTIKEY(0x00000000, 0x04000000, KEY_STAR),
        MULTIKEY(0x00000004, 0x00000000, KEY_SHARP),
        MULTIKEY(0x00000000, 0x00000800, KEY_SOFT1),
        MULTIKEY(0x00000000, 0x00080000, KEY_SOFT2),
        MULTIKEY(0x00000000, 0x08000000, KEY_HOME),
        MULTIKEY(0x00000008, 0x00000000, KEY_BACK),
        MULTIKEY(0x00000020, 0x20000020, KEY_PHONE),
        MULTIKEY(0x00002020, 0x00000020, KEY_MUTE),
        MULTIKEY(0x00000000, 0x20200020, KEY_CAMERA),
        MULTIKEY(0x00000000, 0x00202020, KEY_VOLUMEDOWN),
        MULTIKEY(0x00000000, 0x00002020, KEY_VOLUMEUP),
};
int keymap_size = sizeof(keymap) / sizeof(*keymap);

unsigned int old_state_lo;
unsigned int old_state_hi;

void keypad_scan(int *lo, int *hi)
{
  int col;
  int reg;
  unsigned int state_lo;
  unsigned int state_hi;

  state_lo = state_hi = 0;

  /* read the keypad status */
  REG16(OMAP_MPUIO_BASE + OMAP_MPUIO_KBC) = 0xff;

  for (col = 0; col < 4; col++) {
    REG16(OMAP_MPUIO_BASE + OMAP_MPUIO_KBC) = ~(1 << col);
    udelay(10);
    reg = REG16(OMAP_MPUIO_BASE + OMAP_MPUIO_KBR_LATCH);
    state_lo |= ((unsigned int) ~reg & 0xff) << (8 * col);
  }

  for (col = 4; col < 8; col++) {
    REG16(OMAP_MPUIO_BASE + OMAP_MPUIO_KBC) = ~(1 << col);
    udelay(10);
    reg = REG16(OMAP_MPUIO_BASE + OMAP_MPUIO_KBR_LATCH);
    state_hi |= ((unsigned int) ~reg & 0xff) << (8 * (col - 4));
  }

  REG16(OMAP_MPUIO_BASE + OMAP_MPUIO_KBC) = 0x00;
  udelay(10);

  *lo = state_lo;
  *hi = state_hi;
}

void keypad_init(void)
{
  keypad_scan(&old_state_lo, &old_state_hi);
}

int keypad_get(void)
{
  unsigned int state_lo, state_hi;
  unsigned int change_lo, change_hi;
  unsigned int pending_lo, pending_hi;
  int i;
  int key, pressed;

  do {
    keypad_scan(&state_lo, &state_hi);

    change_lo = state_lo ^ old_state_lo;
    change_hi = state_hi ^ old_state_hi;

    if (change_lo || change_hi) {
     pending_lo = pending_hi = 0;
      for (i = 0; i < keymap_size; i++)
	if (keymap[i].code_lo & change_lo ||
	    keymap[i].code_hi & change_hi) {
	  if((keymap[i].code_lo & change_lo) != keymap[i].code_lo ||
	     (keymap[i].code_hi & change_hi) != keymap[i].code_hi) {
	    pending_lo |= change_lo & keymap[i].code_lo;
	    pending_hi |= change_hi & keymap[i].code_hi;
	  } else {
	    /* all key events received */
	    key = keymap[i].key;
	    pressed = ((state_lo & keymap[i].code_lo) ||
		       (state_hi & keymap[i].code_hi)) ? 1 : 0;
	    pending_lo &= ~keymap[i].code_lo;
	    pending_hi &= ~keymap[i].code_hi;
	    change_lo &= ~keymap[i].code_lo;
	    change_hi &= ~keymap[i].code_hi;
	    old_state_lo = state_lo;
	    old_state_hi = state_hi;

	    if (pressed)
	      return key;
	  }
	}
    }
  } while(1);

  return 0;
}
