/* Embedded OS Loader
 *
 *   Copyright 2005 Vivien Chappelier <vivien.chappelier@free.fr>
 *
 * menu routines
 *
 */

#include "stdlib.h"
#include "fb.h"
#include "font_6x11.h"
#include "list.h"
#include "icon.h"
#include "keypad.h"
#include "conf.h"
#include "menu.h"
#include "console.h"

static struct list menu = { &menu, &menu };
static int selected = -1;
static int menu_count = 0;

void menu_init()
{
  list_init(&menu);
  selected = -1;
  menu_count = 0;
}

int menu_add(int icon_id, char *text)
{
  struct menu_item *item;
  char *icon;
  int len;
  int i, j;

  item = (struct menu_item *) malloc(sizeof(struct menu_item));

  selected = item->id = menu_count++;

  /* copy the item title */
  len = strlen(text) + 1;
  if (len > ITEM_LENGTH - 1)
    len = ITEM_LENGTH - 1;
  memcpy(item->text, text, len);
  item->text[len] = 0;

  /* load the icon */
  switch (icon_id) {
  case ICON_TUX:
    icon = icon_tux;
    break;
  case ICON_WINDOWS:
    icon = icon_windows;
    break;
  default:
    icon = NULL;
    break;
  }

  if (icon) {
    for (j = 0; j < ITEM_LENGTH; j++)
      for (i = 0; i < ITEM_LENGTH; i++)
	item->icon[j][i] =
	  (((int) icon[2 * (j * ITEM_LENGTH + i) + 1]) << 8) |
	   ((int) icon[2 * (j * ITEM_LENGTH + i) + 0]);
  } else {
    for (j = 0; j < ITEM_LENGTH; j++)
      for (i = 0; i < ITEM_LENGTH; i++)
	item->icon[j][i] = MENU_BGCOLOR;
  }

  item->list.prev = menu.prev;
  menu.prev->next = &item->list;
  menu.prev = &item->list;
  item->list.next = &menu;

  // TEMP
  printb("count = %d sel = %d\n", menu_count, selected);

  return selected;
}

int menu_display(void)
{
  struct list *list;
  int x, y;
  int i, j;
  int n;
  char *s;
  fb_pix_t fgcolor, bgcolor, shcolor;
  int fb_width, fb_height;

  fb_width = fb_get_width();
  fb_height = fb_get_height();

  x = 0;
  y = 0;
  n = 0;

  /* fill title background */
  bgcolor = MENU_TITLE_BGCOLOR;
  fgcolor = MENU_FGCOLOR;
  shcolor = MENU_SHCOLOR;

  for (j = 0; j < ITEM_HEIGHT; j++)
    for (i = 0; i < fb_width; i++)
	fb_putpixel(x + i, y + j, bgcolor);

  /* draw the centered text */
  x = (fb_width  - (strlen(MENU_TITLE) * FONTWIDTH)) >> 1;
  s = MENU_TITLE;
  while (*s) {
    char c = *s++;

    if (x + FONTWIDTH + ITEM_TEXT_LEFT_MARGIN + SHADOW_OFFSET < fb_width) {
      fb_draw_char(x + ITEM_TEXT_LEFT_MARGIN + SHADOW_OFFSET,
		   y + ITEM_TEXT_TOP_MARGIN + SHADOW_OFFSET,
		   c, shcolor);
      x += FONTWIDTH;
    }
  }

  x = (fb_width  - (strlen(MENU_TITLE) * FONTWIDTH)) >> 1;
  s = MENU_TITLE;
  while (*s) {
    char c = *s++;

    if (x + FONTWIDTH + ITEM_TEXT_LEFT_MARGIN < fb_width) {
      fb_draw_char(x + ITEM_TEXT_LEFT_MARGIN, y + ITEM_TEXT_TOP_MARGIN,
		   c, fgcolor);
      x += FONTWIDTH;
    }
  }

  y += ITEM_HEIGHT;

  /* draw the title line */
  for (i = 0; i < fb_width; i++)
    fb_putpixel(i, y, fgcolor);
  y++;


  bgcolor = MENU_BGCOLOR;

  for(list = menu.next; list != &menu; list = list->next) {
    struct menu_item *item =
      (struct menu_item *) ((char *) list -
			    (long) &(((struct menu_item *) 0)->list));
    int i, j;
    char *s;

    if (y + ITEM_HEIGHT > fb_height)
      break;

    x = 0;
    
    /* draw the icon */
    for (j = 0; j < ITEM_HEIGHT; j++)
      for (i = 0; i < ITEM_HEIGHT; i++)
	fb_putpixel(x + i, y + j, item->icon[j][i]);

    /* fill the end of line with the background menu color */
    if (n == selected) {
      bgcolor = MENU_SEL_BGCOLOR;
      fgcolor = MENU_SEL_FGCOLOR;
    } else {
      bgcolor = MENU_BGCOLOR;
      fgcolor = MENU_FGCOLOR;
    }

    for (j = 0; j < ITEM_HEIGHT; j++)
      for (i = ITEM_HEIGHT; i < fb_width; i++)
	fb_putpixel(x + i, y + j, bgcolor);

    if (n == selected) {
      fb_putpixel(ITEM_HEIGHT + 0, y + 0, MENU_BGCOLOR);
      fb_putpixel(ITEM_HEIGHT + 0, y + 1, MENU_SEL_HSCOLOR);
      fb_putpixel(ITEM_HEIGHT + 1, y + 0, MENU_SEL_HSCOLOR);
      fb_putpixel(fb_width - 1   , y + 0, MENU_BGCOLOR);
      fb_putpixel(fb_width - 1   , y + 1, MENU_SEL_HSCOLOR);
      fb_putpixel(fb_width - 2   , y + 0, MENU_SEL_HSCOLOR);
      fb_putpixel(ITEM_HEIGHT + 0, y + ITEM_HEIGHT - 1, MENU_BGCOLOR);
      fb_putpixel(ITEM_HEIGHT + 0, y + ITEM_HEIGHT - 2, MENU_SEL_HSCOLOR);
      fb_putpixel(ITEM_HEIGHT + 1, y + ITEM_HEIGHT - 1, MENU_SEL_HSCOLOR);
      fb_putpixel(fb_width - 1   , y + ITEM_HEIGHT - 1, MENU_BGCOLOR);
      fb_putpixel(fb_width - 1   , y + ITEM_HEIGHT - 2, MENU_SEL_HSCOLOR);
      fb_putpixel(fb_width - 2   , y + ITEM_HEIGHT - 1, MENU_SEL_HSCOLOR);
    }

    /* draw the text */
    x += ITEM_HEIGHT;
    s = item->text;
    while (*s) {
      char c = *s++;

      if (x + FONTWIDTH + ITEM_TEXT_LEFT_MARGIN < fb_width) {
	fb_draw_char(x + ITEM_TEXT_LEFT_MARGIN, y + ITEM_TEXT_TOP_MARGIN,
		     c, fgcolor);
	x += FONTWIDTH;
      }
    }

    y += ITEM_HEIGHT;
    n ++;
  }

  return y;
}

int menu_select_down(void) {
  selected ++;
  if (selected >= menu_count)
    selected = menu_count - 1;
  // TEMP
  printb("count = %d\n", menu_count);
  return selected;
}

int menu_select_up(void) {
  selected --;
  if (selected < 0)
    selected = 0;
  return selected;
}

int menu_select(int sel) {
  if (sel >= 0 && sel < menu_count)
    selected = sel;
  return selected;
}

void softkey_display(char *lefttext, char *righttext)
{
  int x, y;
  int j;
  char *s;
  fb_pix_t fgcolor, bgcolor, shcolor;
  int fb_width, fb_height;

  fb_width = fb_get_width();
  fb_height = fb_get_height();

  x = 0;
  y = fb_height - FONTHEIGHT;

  bgcolor = MENU_SOFTKEY_BGCOLOR;
  fgcolor = MENU_FGCOLOR;
  shcolor = MENU_SHCOLOR;

  /* draw the background */
  fb_fill(0, fb_height - FONTHEIGHT - 1, fb_width, 1, fgcolor);
  fb_fill(0, fb_height - FONTHEIGHT, fb_width, FONTHEIGHT, bgcolor);

  for (j = 0; j < FONTHEIGHT; j++)
    fb_putpixel(fb_width >> 1, fb_height - FONTHEIGHT + j, fgcolor);

  /* draw the left softkey */
  x = ((fb_width >> 1)  - (strlen(lefttext) * FONTWIDTH)) >> 1;
  s = lefttext;
  while (*s) {
    char c = *s++;

    if (x + FONTWIDTH + SHADOW_OFFSET < fb_width) {
      fb_draw_char(x + SHADOW_OFFSET, y + SHADOW_OFFSET, c, shcolor);
      x += FONTWIDTH;
    }
  }

  x = ((fb_width >> 1)  - (strlen(lefttext) * FONTWIDTH)) >> 1;
  s = lefttext;
  while (*s) {
    char c = *s++;

    if (x + FONTWIDTH < fb_width) {
      fb_draw_char(x, y, c, fgcolor);
      x += FONTWIDTH;
    }
  }

  /* draw the right softkey */
  x = (fb_width >> 1) +
    (((fb_width >> 1)  - (strlen(righttext) * FONTWIDTH)) >> 1);
  s = righttext;
  while (*s) {
    char c = *s++;

    if (x + FONTWIDTH + SHADOW_OFFSET < fb_width) {
      fb_draw_char(x + SHADOW_OFFSET, y + SHADOW_OFFSET, c, shcolor);
      x += FONTWIDTH;
    }
  }

  x = (fb_width >> 1) +
    (((fb_width >> 1)  - (strlen(righttext) * FONTWIDTH)) >> 1);
  s = righttext;
  while (*s) {
    char c = *s++;

    if (x + FONTWIDTH < fb_width) {
      fb_draw_char(x, y, c, fgcolor);
      x += FONTWIDTH;
    }
  }
}

struct confentry *menu_loop(struct conf *conf)
{
  struct list *list;
  int console_top;
  int sel = -1;
  int i;

  /* prepare the menu */
  menu_init();

  for_list(&conf->entries, list) {
    struct confentry *entry = list_item(list, struct confentry, list);
    sel = menu_add(entry->icon, entry->title);
  }

  /* clear the fb */
  fb_clear(MENU_BGCOLOR);

  /* resize the console */
  if (console_visible())
	  console_hide();
  console_top = fb_get_height() - FONTHEIGHT - 1 - 5 * FONTHEIGHT;
  console_resize(0, console_top, fb_get_width(),
		 fb_get_height() - console_top - FONTHEIGHT - 1);

  keypad_init();
  menu_display();
  softkey_display("Boot", "Show Console");

  do {
    int key;

    key = keypad_get();

    switch(key) {

    case KEY_VOLUMEUP:
      /* TODO: scroll console up */
      break;

    case KEY_VOLUMEDOWN:
      /* TODO: scroll console down */
      break;

    case KEY_SOFT1:
      i = 0;
      for_list(&conf->entries, list) {
	struct confentry *entry = list_item(list, struct confentry, list);
	if (i == sel)
	  return entry;
	i++;
      }
      return NULL;

    case KEY_SOFT2:
      if (console_visible()) {
	console_hide();
	fb_clear(MENU_BGCOLOR);
	menu_display();
	softkey_display("Boot", "Show Console");
      } else {
	console_show();
	softkey_display("Boot", "Hide Console");
      }
      break;

    case KEY_DOWN:
      sel = menu_select_down();
      menu_display();
      break;

    case KEY_UP:
      sel = menu_select_up();
      menu_display();
      break;

    case KEY_1:
      sel = menu_select(0);
      menu_display();
      break;
    case KEY_2:
      sel = menu_select(1);
      menu_display();
      break;
    case KEY_3:
      sel = menu_select(2);
      menu_display();
      break;
    case KEY_4:
      sel = menu_select(3);
      menu_display();
      break;
    case KEY_5:
      sel = menu_select(4);
      menu_display();
      break;
    case KEY_6:
      sel = menu_select(5);
      menu_display();
      break;
    case KEY_7:
      sel = menu_select(6);
      menu_display();
      break;
    case KEY_8:
      sel = menu_select(7);
      menu_display();
      break;
    case KEY_9:
      sel = menu_select(8);
      menu_display();
      break;
    }
  } while(1);

  return NULL;
}
