/* Embedded OS Loader
 *
 *   Copyright 2005 Vivien Chappelier <vivien.chappelier@free.fr>
 *
 * standard routines
 *
 */

#include <stdarg.h>
#include "stdlib.h"
#include "console.h"

unsigned int timer(void) /* decreases */
{
  return(REG32(OMAP730_TIMER_BASE));
}

void udelay(int timeout)
{
  unsigned int start;

  start = timer();
  while(start - timer() <= ((timeout * 7) >> 3) + 1);
}

void mdelay(int timeout)
{
  int i;

  for(i = 0; i < 1000; i++)
    udelay(timeout);
}

void die(void)
{
  while(1);
}

extern void *malloc_base;
void *malloc(size_t size)
{
  void *ptr = malloc_base;
  malloc_base += (size + 3) & ~3; /* keep aligned */
  //printb("malloc: 0x%x\n", ptr);
  return(ptr);
}

void *memset(void *buffer, int val, size_t length)
{
  unsigned char *mem = (unsigned char *) buffer;
  int i;
  
  for(i = 0; i < length; i++)
    mem[i] = (unsigned char) (val & 0xff);
  return(buffer);
}

void *memcpy(void *output, const void *input, size_t length)
{
  const unsigned char *in = (const unsigned char *) input;
  unsigned char *out = (unsigned char *) output;
  int i;
  
  for(i = 0; i < length; i++)
    out[i] = in[i];
  return(output);
}

size_t strlen(const char *str)
{
  int i;

  for(i = 0; *str++; i++);
  return(i);
}

char const *strchr(char const *str, int c)
{
  while(*str) {
    if(*str == c) return(str);
    str++;
  }
  return(NULL);
}

char const *strrchr(char const *str, int c)
{
  char const *last = str + strlen(str) - 1;

  while(last >= str) {
    if(*last == c) return(last);
    last--;
  }
  return(NULL);
}

char *strdup(char *str)
{
  char *s;

  s = (char *) malloc(strlen(str) + 1);
  memcpy(s, str, strlen(str) + 1);
  return s;
}

int __strncmp(const char *str1, const char *str2, size_t count)
{
  int i;
  
  for(i = 0; i < count; i++) {
    if(*str1 < *str2) return(-1);
    if(*str1 > *str2) return(+1);
    if(*str1 == 0) return(0);
    str1++;
    str2++;
  }
  return(0);
}

int __strncasecmp(const char *str1, const char *str2, size_t count)
{
  int i;
  char c1, c2;
  
  for(i = 0; i < count; i++) {
    c1 = *str1;
    c2 = *str2;
    if (c1 >= 'A' && c1 <= 'Z')
      c1 |= 0x20;
    if (c2 >= 'A' && c2 <= 'Z')
      c2 |= 0x20;
    if(c1 < c2) return(-1);
    if(c1 > c2) return(+1);
    if(c1 == 0) return(0);
    str1++;
    str2++;
  }
  return(0);
}

unsigned int __udiv(unsigned int a, unsigned int b)
{
  unsigned int i = 0;

  while(a >= b) {
    a -= b;
    i++;
  }

  return(i);
}

int puts(const char *string)
{
  while(*string) putchar(*string++);
  putchar('\n');
  return(0);
}

static const unsigned int dec[] = { 1000000000, 100000000, 10000000, 1000000, 100000, 10000, 1000, 100, 10, 1, 0 };

void printb(const char *format, ...)
{
  va_list ap;
  int d;
  int i;
  char c, *str;
  int f;
  unsigned int u;

  va_start(ap, format);
  while(1) {
    f = *format++;
    if(!f) break;

    if(f == '%') { /* special char */
      f = *format++;
      if(!f) break;

      switch(f) {
      case 's':           /* string */
	str = va_arg(ap, char *);
	while(*str) putchar(*str++);
	break;
      case 'c':           /* char */
	c = (char) va_arg(ap, int);
	putchar(c);
	break;
      case 'd':           /* int */
      case 'u':
	d = va_arg(ap, int);
	if(f == 'd' && d < 0) {
	  putchar('-');
	  u = -d;
	} else
	  u = (unsigned) d;

	if(!u) {
	  putchar('0');
	  break;
	}

	for(i = 0; u < dec[i]; i++);
	while(dec[i]) {
	  unsigned int r = __udiv(u, dec[i]);
	  putchar('0' + r);
	  u -= r * dec[i];
	  i++;
	}
	break;
      case 'x':           /* int (hexa) */
	u = va_arg(ap, unsigned int);
	if(!u) {
	  putchar('0');
	  break;
	}
	i = 28;
	while(!(u >> i)) i -= 4;
	while(i >= 0) {
	  if(((u >> i) & 0xf) > 9)
	    putchar('a' + ((u >> i) & 0xf) - 10);
	  else
	    putchar('0' + ((u >> i) & 0xf));
	  i -= 4;
	}
	break;
      case 'p':           /* pointer */
	u = (int) va_arg(ap, void *); /* 32 bit */
	putchar('0');
	putchar('x');
	i = 28;
	while(i >= 0) {
	  if(((u >> i) & 0xf) > 9)
	    putchar('a' + ((u >> i) & 0xf) - 10);
	  else
	    putchar('0' + ((u >> i) & 0xf));
	  i -= 4;
	}
	break;
      default:            /* unknown format */
	putchar('?');
	break;
      }
    } else {       /* standard char */
      putchar(f);
    }
  }

  va_end(ap);
}

